@@ -6,18 +6,19 @@ import json
 class MessageTypes(object):
     SHUTDOWN, LOG_EVENT = range(1, 3)
 
-class Message(object):
 
+class Message(object):
     def __init__(self, msg_type, payload):
         self.msg_type = msg_type
         self.payload = payload
 
     def serialize(self):
         msg = {
-            'msg_type': self.msg_type,
-            'payload': self.payload,
+            "msg_type": self.msg_type,
+            "payload": self.payload,
         }
-        return json.dumps(msg)+"\n"
+        return json.dumps(msg) + "\n"
+
 
 def deserialize(json_msg):
-    return Message(**json.loads(json_msg))
\ No newline at end of file
+    return Message(**json.loads(json_msg))
